package simple;

import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlSerializer;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import ru.vfilippov.electronreminder.Trip;

public class SimpleXml
{
    public static SimpleXmlNode parse(InputStream in) throws IOException, XmlPullParserException
    {
        try
        {
            XmlPullParser parser = Xml.newPullParser();
            parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
            parser.setInput(in, null);
            return readFeed(parser);
        }
        finally
        {
            in.close();
        }
        /**
         import javax.xml.parsers.DocumentBuilder;
         import javax.xml.parsers.DocumentBuilderFactory;
         import javax.xml.parsers.ParserConfigurationException;

         Document d = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(bis);
         NodeList segments = d.getElementsByTagName("segment");
         segments.item(0).getAttributes().getNamedItem("from")
         catch (SAXException e)
         {
             e.printStackTrace();
         }
         catch (ParserConfigurationException e)
         {
             e.printStackTrace();
         }
         */
    }

    protected static void writeNode(SimpleXmlNode n, XmlSerializer s) throws IOException
    {
        s.startTag("", n.name);
        for (String key: n.attributes.keySet())
            s.attribute("", key, n.attributes.get(key));
        if (n.value != null && n.value.length() > 0)
            s.text(n.value);
        for (SimpleXmlNode child: n.children)
            writeNode(child, s);
        s.endTag("", n.name);
    }

    public static void write(SimpleXmlNode n, OutputStream out) throws IOException
    {
        XmlSerializer s = Xml.newSerializer();
        s.setOutput(out, "utf-8");
        s.startDocument("utf-8", true);
        writeNode(n, s);
        s.endDocument();
    }

    public static SimpleXmlNode readXmlFile(File dir, String name)
    {
        File file = new File(dir, name);
        SimpleXmlNode n = null;
        if (file.exists())
        {
            FileInputStream is = null;
            try
            {
                is = new FileInputStream(file);
                n = SimpleXml.parse(is);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            catch (XmlPullParserException e)
            {
                e.printStackTrace();
            }
            finally
            {
                try
                {
                    is.close();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
        }
        return n;
    }

    public static SimpleXmlNode readFeed(XmlPullParser parser) throws IOException, XmlPullParserException
    {
        ArrayList<SimpleXmlNode> stack = new ArrayList<SimpleXmlNode>();
        while (parser.next() != XmlPullParser.END_DOCUMENT)
        {
            if (parser.getEventType() == XmlPullParser.START_TAG)
            {
                SimpleXmlNode node = new SimpleXmlNode();
                node.name = parser.getName();
                for (int i = 0; i < parser.getAttributeCount(); i++)
                {
                    String p = parser.getAttributePrefix(i);
                    if (p == null)
                        p = "";
                    node.attributes.put(p + parser.getAttributeName(i), parser.getAttributeValue(i));
                }
                if (stack.size() > 0)
                    stack.get(stack.size()-1).appendChild(node);
                stack.add(node);
            }
            else if (parser.getEventType() == XmlPullParser.END_TAG)
            {
                if (stack.size() > 1)
                    stack.remove(stack.size() - 1);
                else
                    return stack.get(0);
            }
            else if (parser.getEventType() == XmlPullParser.TEXT)
            {
                SimpleXmlNode n = stack.get(stack.size() - 1);
                n.value = (n.value == null ? parser.getText() : n.value+parser.getText());
            }
        }
        return null;
    }
}
